pub struct CapacityAllocator { /* private fields */ }Expand description
A reusable capacity allocator for object counts, bytes, or other units.
Implementations
Sourceimpl CapacityAllocator
impl CapacityAllocator
Sourcepub fn new(
parent: &Rc<Entity>,
name: &str,
capacity: usize,
capacity_unit: impl Into<String>,
) -> Result<Self, SimError>
pub fn new( parent: &Rc<Entity>, name: &str, capacity: usize, capacity_unit: impl Into<String>, ) -> Result<Self, SimError>
Create a standalone allocator with its own child entity.
Sourcepub fn for_entity(
entity: &Rc<Entity>,
capacity: usize,
capacity_unit: impl Into<String>,
) -> Result<Self, SimError>
pub fn for_entity( entity: &Rc<Entity>, capacity: usize, capacity_unit: impl Into<String>, ) -> Result<Self, SimError>
Create allocator bookkeeping on an existing entity.
pub fn used(&self) -> usize
pub fn capacity(&self) -> usize
pub fn capacity_unit(&self) -> &str
pub fn level_change_event(&self) -> Repeated<usize>
pub async fn wait_for_capacity(&self, units: usize) -> SimResult
pub fn allocate(&self, units: usize) -> SimResult
pub fn release(&self, units: usize)
pub async fn reserve( &self, units: usize, ) -> Result<CapacityReservation, SimError>