ByteStore

Struct ByteStore 

pub struct ByteStore<T>(/* private fields */);
Builds stores that support a configurable number of bytes.

Objects must support the [SimObject] trait. The returned Store uses total_bytes to decide whether there is enough free byte capacity to accept each object.

Implementations

Source

impl<T> ByteStore<T>
where T: SimObject,

pub fn new_and_register_with_renames( engine: &Engine, clock: &Clock, parent: &Rc<Entity>, name: &str, aka: Option<&Aka>, capacity_bytes: usize, ) -> Result<Rc<Store<T>>, SimError>

Basic byte-store constructor.

Returns a SimError if capacity_bytes is 0.

pub fn new_and_register( engine: &Engine, clock: &Clock, parent: &Rc<Entity>, name: &str, capacity_bytes: usize, ) -> Result<Rc<Store<T>>, SimError>

Basic byte-store constructor.

Returns a SimError if capacity_bytes is 0.