Queue

Struct Queue 

pub struct Queue<T>
where T: SimObject,
{ /* private fields */ }
A generic queue for simulation objects.

Implementations

Source

impl<T> Queue<T>
where T: SimObject,

pub fn new( parent: &Rc<Entity>, name: &str, capacity: Option<usize>, ) -> Result<Self, SimError>

Create a new queue.

Returns a [SimError] if capacity is Some(0).

pub fn len(&self) -> usize

Return the current queue length.

pub fn is_empty(&self) -> bool

Return whether the queue is empty.

pub fn is_full(&self) -> bool

Return whether the queue is full.

pub fn values(&self) -> Vec<T>

Return a snapshot of the queue contents by copying all values.

pub fn changed_event(&self) -> Repeated<()>

Return an event that fires whenever the queue contents change.

pub async fn push(&self, value: T) -> SimResult

Push a value into the queue, waiting until there is space if required.

pub fn pop_front(&self) -> Option<T>

Pop the oldest value from the queue.

pub fn remove_where<F>(&self, predicate: F) -> Option<T>
where F: FnMut(&T) -> bool,

Remove the first value matching the predicate.