pub struct ByteStore<T>(/* private fields */);Expand description
Builds stores that support a configurable number of bytes.
Objects must support the [SimObject] trait. The returned Store uses
total_bytes to decide
whether there is enough free byte capacity to accept each object.
Implementations
Sourceimpl<T> ByteStore<T>where
T: SimObject,
impl<T> ByteStore<T>where
T: SimObject,